// code by jph
package sys.col;

import java.awt.Color;

import sys.mat.Fraction;

/** taken from stackoverflow
 * 
 * Perceived brightness? Luminance?
 * (standard for certain color spaces): (0.2126*R + 0.7152*G + 0.0722*B)
 * (perceived option 1): (0.299*R + 0.587*G + 0.114*B)
 * (perceived option 2, slower to calculate): sqrt( 0.241*R^2 + 0.691*G^2 + 0.068*B^2 ) ? sqrt( 0.299*R^2 + 0.587*G^2 + 0.114*B^2 )
 * 
 * I think what you are looking for is the RGB -> Luma conversion formula.
 * 
 * Photometric/digital ITU-R:
 * Y = 0.2126 R + 0.7152 G + 0.0722 B
 * 
 * Digital CCIR601 (gives more weight to the R and B components):
 * Y = 0.299 R + 0.587 G + 0.114 B
 * 
 * If you are willing to trade accuracy for perfomance, there are two approximation formulas for this one:
 * Y = 0.33 R + 0.5 G + 0.16 B
 * Y = 0.375 R + 0.5 G + 0.125 B
 * 
 * These can be calculated quickly as
 * Y = (R+R+B+G+G+G)/6
 * Y = (R+R+R+B+G+G+G+G)>>3 */
public class LumaPalette extends Palette {
  // { 0.30, 0.59, 0.11 };
  // { 0.40, 0.55, 0.25 };
  // { 0.45, 0.50, 0.35 };
  static final double[] brighten_weights = new double[] { 0.45, 0.50, 0.4 };
  public static final Palette default13 = new LumaPalette(HuePalette.prime(13, new Fraction(2, 5)), brighten_weights);
  // ---
  final Palette myPalette;
  final double[] values;
  final double[] weight;
  double max = 0;
  double min = 1e7;

  // public static LumaPalette createPrime(int size) {
  // return new LumaPalette(HuePalette.prime(size, new Fraction(2, 5)), brighten_weights);
  // }
  public LumaPalette(Palette myPalette, double[] weight) {
    this.myPalette = myPalette;
    // ---
    values = new double[size()];
    this.weight = weight;
    int index = 0;
    for (Color myColor : myPalette) {
      values[index] = fakeITU(myColor);
      // System.out.println();
      // max = Math.max(max, values[index]);
      min = Math.min(min, values[index]);
      ++index;
    }
  }

  private double fakeITU(Color myColor) {
    int r = myColor.getRed();
    int g = myColor.getGreen();
    int b = myColor.getBlue();
    int r2 = r * r;
    int g2 = g * g;
    int b2 = b * b;
    return Math.sqrt(weight[0] * r2 + weight[1] * g2 + weight[2] * b2) / Math.sqrt(r2 + g2 + b2);
  }

  @Override
  public Color getColor(int index, double saturation, double value, double alpha) {
    // if (index < values.length)
    return myPalette.getColor(index, saturation, value * Math.pow(min / values[index], saturation), alpha);
    // return new Color(0, 0, 0, (float) alpha);
  }

  @Override
  public int size() {
    return myPalette.size();
  }
  
  public static void main(String[] args) {
    default13.printTable();
  }
}
